from PyQt4.QtGui import QComboBox, QLabel, QHBoxLayout, QSpinBox, QVBoxLayout

from qgis.core import QgsFeatureRendererV2, QgsRendererV2AbstractMetadata
from qgis.gui import QgsRendererV2Widget

from utils import symbolForCode, getDefaultSymbol


class MyTestRenderer(QgsFeatureRendererV2):
    def __init__(self, size=40, field='', fields=[]):
        QgsFeatureRendererV2.__init__(self, 'MyTestRenderer')
        self.field = field
        self.fields = fields
        self.size = size
        self.cachedSymbols = {}

    def symbolForFeature(self, feature):
        idx = feature.fieldNameIndex(self.field)
        if idx != -1:
            code = feature.attributes()[idx]
            if code in self.cachedSymbols:
                return self.cachedSymbols[code]
            symbol = getDefaultSymbol(self.size)
            self.cachedSymbols[code] = symbol
            return symbol
        else:
            return getDefaultSymbol(self.size)

    def startRender(self, context, vlayer):
        for s in self.cachedSymbols.values():
            s.startRender(context)

    def stopRender(self, context):
        for s in self.cachedSymbols.values():
            s.stopRender(context)

    def symbols2(self, context):
        return self.cachedSymbols.values()

    def usedAttributes(self):
        return self.fields

    def clone(self):
        return MyTestRenderer(self.size, self.field, self.fields)

    def save(self, doc):
        elem = doc.createElement('renderer-v2')
        elem.setAttribute('type', 'MyTestRenderer')
        elem.setAttribute('size', self.size)
        elem.setAttribute('field', self.field)

        return elem

    def create(self, element):
        size = int(element.attribute('size'))
        field = element.attribute('field')

        r = MyTestRenderer(size, field)
        return r


class MyTestRendererWidget(QgsRendererV2Widget):
    def __init__(self, layer, style, renderer):
        QgsRendererV2Widget.__init__(self, layer, style)

        if renderer is None or renderer.type() != 'MyTestRenderer':
            fields = [f.name() for f in layer.dataProvider().fields()]
            self.r = MyTestRenderer(field=fields[0], fields=fields)
        else:
            self.r = renderer

        self.combo = QComboBox()
        for f in layer.dataProvider().fields():
            self.combo.addItem(f.name(), f.name())

        idx =  max(0, layer.dataProvider().fieldNameIndex(self.r.field))

        self.combo.setCurrentIndex(idx)
        self.combo.currentIndexChanged.connect(self.fieldChanged)

        self.labelField = QLabel('Code field')
        self.hbox = QHBoxLayout()
        self.hbox.addWidget(self.labelField)
        self.hbox.addWidget(self.combo)

        self.spinSize = QSpinBox()
        self.spinSize.setValue(self.r.size)
        self.spinSize.valueChanged.connect(self.sizeChanged)

        self.labelSize = QLabel('Size (pixels)')
        self.hbox2 = QHBoxLayout()
        self.hbox2.addWidget(self.labelSize)
        self.hbox2.addWidget(self.spinSize)
        self.vbox = QVBoxLayout()
        self.vbox.addLayout(self.hbox)
        self.vbox.addLayout(self.hbox2)
        self.setLayout(self.vbox)

    def sizeChanged(self, value):
        self.r.size = value

    def fieldChanged(self):
        self.r.field = self.combo.currentText()

    def renderer(self):
        return self.r


class MyTestRendererMetadata(QgsRendererV2AbstractMetadata):
    def __init__(self):
        QgsRendererV2AbstractMetadata.__init__(self, 'MyTestRenderer', 'My test renderer')

    def createRenderer(self, element):
        self.renderer = MyTestRenderer()
        return self.renderer

    def createRendererWidget(self, layer, style, renderer):
        return MyTestRendererWidget(layer, style, renderer)
